'''
Provide sample API mocking customized HTTP status codes.
'''
from .contrib import bottle

STATUS_LINES = {
    100: 'Continue',
    101: 'Switching Protocols',
    200: 'OK',
    201: 'Created',
    202: 'Accepted',
    203: 'Non-Authoritative Information',
    204: 'No Content',
    205: 'Reset Content',
    206: 'Partial Content',
    300: 'Multiple Choices',
    301: 'Moved Permanently',
    302: 'Found',
    303: 'See Other',
    304: 'Not Modified',
    305: 'Use Proxy',
    307: 'Temporary Redirect',
    400: 'Bad Request',
    401: 'Unauthorized',
    402: 'Payment Required',
    403: 'Forbidden',
    404: 'Not Found',
    405: 'Method Not Allowed',
    406: 'Not Acceptable',
    407: 'Proxy Authentication Required',
    408: 'Request Time-out',
    409: 'Conflict',
    410: 'Gone',
    411: 'Length Required',
    412: 'Precondition Failed',
    413: 'Request Entity Too Large',
    414: 'Request-URI Too Large',
    415: 'Unsupported Media Type',
    416: 'Requested range not satisfiable',
    417: 'Expectation Failed',
    418: 'I\'m a teapot',
    428: 'Precondition Required',
    429: 'Too Many Requests',
    431: 'Request Header Fields Too Large',
    500: 'Internal Server Error',
    501: 'Not Implemented',
    502: 'Bad Gateway',
    503: 'Service Unavailable',
    504: 'Gateway Time-out',
    505: 'HTTP Version not supported',
    511: 'Network Authentication Required'}


@bottle.route('/_tb/status/<status>')
def status_code(status=''):
    try:
        status = int(status) if int(status) in STATUS_LINES else 400
    except Exception:
        status = 400
    return bottle.HTTPResponse(status=status,
                               status_line=STATUS_LINES.get(status, ''))



# coding: utf-8
'''
utility functions.
'''
from wsgiref.util import is_hop_by_hop
try:
    from urllib.parse import urlparse
except ImportError:
    from urlparse import urlparse
import six


def format_prefix(prefix):
    '''
    Format url path prefix, ensure '/' in the begining.
    '''
    prefix = '' if prefix is None else prefix
    prefix = prefix if prefix.startswith('/') else '/%s' % prefix
    return prefix


def filter_request_headers(req):
    ret = [(k, v) for (k, v) in req.headers.items()\
           if not is_hop_by_hop(k)]
    remove_headers = ['content-length', 'host']
    ret = dict(filter(lambda x: x[0].lower() not in remove_headers,
                      ret))
    return ret


def replace_location_host(location, upstream, request_host):
    parsed_location = urlparse(location)
    parsed_upstream = urlparse(upstream)
    if parsed_location.netloc and parsed_upstream.netloc\
            and parsed_location.scheme == parsed_upstream.scheme\
            and parsed_upstream.netloc.lower() in parsed_location.netloc.lower():
        return location.lower().replace(parsed_upstream.netloc.lower(),
                                        request_host)
    else:
        return location


def concat_path(*parts):
    ret = ''
    for i in parts:
        if ret.endswith('/') and i.startswith('/'):
            ret = ret[:-1] + i
        else:
            ret = ret + i
    return ret


def to_unicode(content, encoding='utf-8'):
    if hasattr(content, 'read'):
        content = content.read()
    if not isinstance(content, six.text_type):
        content = content.decode(encoding, 'ignore')
    return content


def to_bytes(content, encoding='utf-8'):
    if isinstance(content, six.text_type):
        content = content.encode(encoding, 'ignore')
    return content


def insert_adjacent_html(element, position, html):
    import bs4
    new_element = bs4.BeautifulSoup(html, 'html.parser')
    if position == "beforebegin":
        element.insert_before(new_element)
    elif position == "afterbegin":
        element.insert(0, new_element)
    elif position == "beforeend":
        element.append(new_element)
    elif position == "afterend":
        element.insert_after(new_element)
    else:
        pass


def dom_insert_adjacent_html(dom_html, selector, position, html):
    '''
    对于在dom_html中，通过selector选定的标签，在指定的position中插入的html。
    此方法的机制类似于js中的insertAdjacentHTML方法，
    参考 https://developer.mozilla.org/en-US/docs/Web/API/Element/insertAdjacentHTML
    '''
    import bs4
    dom = bs4.BeautifulSoup(dom_html, 'html.parser')
    for element in dom.select(selector):
        insert_adjacent_html(element, position, html)
    return str(dom)



import sys
import argparse
import importlib
import six
from .contrib import bottle


def parse_args():
    parser = argparse.ArgumentParser()
    parser.add_argument("--port", default=8080,
                        help="Port to bind on.")
    parser.add_argument("--host", default="0.0.0.0",
                        help="Host to bind.")
    parser.add_argument("--echo", action="store_true",
                        help="Module echo, return the current http request detail.")
    parser.add_argument("--echo-prefix", default="/_y3k/",
                        help="The path prefix for echo module")
    parser.add_argument("--mock",
                        help="Enable mocking data via a json file.")
    parser.add_argument("--mock-prefix", default="",
                        help="The path prefix for all mocking data.")
    parser.add_argument("--static", default="",
                        help="Serve static files with specified directory.")
    parser.add_argument("--static-prefix", default="/",
                        help="The path prefix for serving static files with --static.")
    parser.add_argument("--static-serve-dir", action="store_true",
                        help="Serve directories as a list.")
    parser.add_argument("--zip", default="",
                        help="Serve a zip file as static directory.")
    parser.add_argument("--zip-prefix", default="/",
                        help="The path prefix for the serving file with --zip.")   
    parser.add_argument("--zip-encoding", default="utf-8",
                        help="The encoding of zipfile")   
    parser.add_argument("--interact", action="store_true",
                        help="Attach to a interactive console.")
    parser.add_argument("--interact-path", default="/_y3k/interact",
                        help="The path for interactive console.")
    parser.add_argument("--server", default="auto",
                        help="Specify the web server for running ynm3k, "
                        "options available at https://bottlepy.org/docs/dev/deployment.html#switching-the-server-backend")
    parser.add_argument("--version", action='store_true', default=False,
                        help="Show version and exit.")
    args = parser.parse_args()
    return args


def main():
    args = vars(parse_args())
    if args['version']:
        from . import __VERSION__
        print("ynm3k %s" % __VERSION__)
        return
    if args['echo']:
        from . import echo
        object_echo = echo.ModuleEcho(args['echo_prefix'])
    if args['interact']:
        from . import interact
        object_interact = interact.ModuleInteract(args['interact_path'])
    if args['static']:
        from . import static
        object_static = static.ModuleStatic(args['static_prefix'],
                                            path=args['static'],
                                            serve_dir=args['static_serve_dir'])
    if args['zip']:
        from .modules import zip_serve
        object_zip = zip_serve.ModuleZipServe(args['zip_prefix'],
                                              args['zip'],
                                              args['zip_encoding'])
    if args['mock']:
        from . import mock
        object_mock = mock.ModuleMock(args['mock_prefix'], args['mock'])    
    bottle.run(host=args['host'], port=int(args['port']),
               debug=True, server=args["server"])


if __name__ == '__main__':
    if six.PY2:
        reload(sys)
        sys.setdefaultencoding('utf-8')
    main()



#!/usr/bin/env python
from ynm3k import __VERSION__

long_description = ""

try:
    import pypandoc
    long_description = pypandoc.convert('README.md', 'rst')
except:
    pass

sdict = {
    'name': 'ynm3k',
    'version': __VERSION__,
    'packages': ['ynm3k',
                 'ynm3k.contrib',
                 'ynm3k.modules'],
    'scripts': ['y3k'],
    'zip_safe': False,
    'install_requires': ['six', 'requests', 'hjson'],
    'author': 'Lichun',
    'long_description': long_description,
    'url': 'https://github.com/socrateslee/ynm3k',
    'classifiers': [
        'Environment :: Console',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python']
}

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

if __name__ == '__main__':
    setup(**sdict)



# coding:utf-8
'''
Module for serving a qiniu object storage.
'''
import hmac
import hashlib
import base64
import six
import requests
from . import util


class QiniuClient(object):
    HOST = "https://rs.qbox.me"

    def __init__(self, ak, sk):
        self.ak = six.b(ak) if isinstance(ak, six.text_type) else ak
        self.sk = six.b(sk) if isinstance(sk, six.text_type) else sk
        self.session = requests.Session()

    def request(self, method, path, **kw):
        req = requests.Request(method, "%s%s" % (self.HOST, path), **kw)
        preq = req.prepare()
        sign_str = six.b(preq.url.replace(self.HOST, '') + '\n' + (preq.body or ''))
        encode_sign = base64.urlsafe_b64encode(hmac.new(self.sk, sign_str, hashlib.sha1).digest())
        access_token = b"%s:%s" % (self.ak, encode_sign)
        print(access_token)
        preq.headers['Authorization'] = b"QBox %s" % access_token
        return self.session.send(preq)


class ModuleQiniuServe(object):
    @classmethod
    def from_resp_spec(cls, req_spec, resp_spec, **kw):
        pass

    def __init__(self, prefix, key_prefix, bucket, ak, sk, bind=True, **kw):
        self.prefix = util.format_prefix(prefix)
        self.key_prefix = key_prefix




